// UtilityConfig.cpp
//
// Copyright (C) 2011, ARM Research Labs, LLC.
// See www.armresearch.com for the copyright terms.
//
// This file implements the common functionality for the configuration
// utilities.

#include <cerrno>
#include <cstring>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <stdexcept>
#include <sstream>
#include <iostream>
#include <fstream>
#include <vector>

#include "UtilityConfig.hpp"

using namespace std;

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// Configuration. ////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

// Initialize sample ignore list file path.
#ifdef WIN

//  Windows OS.
const std::string UtilityConfig::SampleIgnoreListFile("C:\\SNF\\GBUdbIgnoreList.txt.sample");

#else

#ifdef DEFAULT_DATA_DIR
//  *nix, DEFAULT_DATA_DIR is specified on the compile command line.
const std::string UtilityConfig::SampleIgnoreListFile(DEFAULT_DATA_DIR "/GBUdbIgnoreList.txt.sample");
#else
// Not Windows, and DEFAULT_DATA_DIR is not specified on the compile
// command line.  In this case, we don't know the path for the sample
// ignore list file.
#error DEFAULT_DATA_DIR must be defined by -DDEFAULT_DATA_DIR="..." when compiling.
#endif

#endif

// Initialize sample rulebase script file.
#ifdef WIN

//  Windows OS.
const std::string UtilityConfig::SampleRulebaseScriptFile("C:\\SNF\\getRulebase.sample");

#else

#ifdef SBIN_DIR
//  *nix, SBIN_DIR is specified on the compile command line.
const std::string UtilityConfig::SampleRulebaseScriptFile(SBIN_DIR "/getRulebase.sample");
#else
// Not Windows, and SBIN_DIR is not specified on the compile
// command line.  In this case, we don't know the path for the sample
// ignore list file.
#error SBIN_DIR must be defined by -DSBIN_DIR="..." when compiling.
#endif

#endif

// Initialize OS-specific values.
#ifdef WIN
//  Windows OS.
const std::string UtilityConfig::OperatingSystemType("Windows");
#else
#ifdef SNF_OSTYPE
//  *nix, SNF_OSTYPE is specified on the compile command line.
const std::string UtilityConfig::OperatingSystemType(SNF_OSTYPE);
#else
// Not Windows, and SNF_OSTYPE is not specified on the compile command
// line.  In this case, we don't know the operating system.
#error SNF_OSTYPE must be defined by -DSNF_OSTYPE="..." when compiling.
#endif
#endif

/// Verbose command-line input.
const string VerboseKey("-v");

/// Explain command-line input.
const string ExplainKey("-explain");

/// Help command-line input.
const string HelpKey("-h");

//////////////////////////////////////////////////////////////////////////////////////////////////////////
// End of configuration. /////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////

UtilityConfig::UtilityConfig() {

    SetExplain(false);
    SetVerbose(false);
    SetHelp(false);

}    

void
UtilityConfig::CheckAndLoadConfigFile(const std::string DefaultFile[], int NumDefaultFiles) {

    string ProvisionalConfigFile = ConfigFile;

    if (ProvisionalConfigFile.length() == 0) {

	int i;
	vector<string> FoundFile;

	for (i = 0; i < NumDefaultFiles; i++) {

	    if (!FileExists(DefaultFile[i])) {

                continue;                                                       // File doesn't exist.

	    }

	    FoundFile.push_back(DefaultFile[i]);                                // Update list of found files.
	    ProvisionalConfigFile = DefaultFile[i];                             // Found configuration file.

	}

	if (0 == FoundFile.size()) {                                            // No default file found.

	    string Temp;

	    Temp = "Configuration file was not specified, and no default configuration file was found.  ";
	    Temp += "Checked:\n\n";

	    for (i = 0; i < NumDefaultFiles; i++) {

		Temp += "  ";
		Temp += DefaultFile[i] + "\n";

	    }

	    throw runtime_error(Temp);

	}

        if (FoundFile.size() > 1) {                                             // No default file found.

	    string Temp;

	    Temp = "Configuration file was not specified, and more than one default configuration file was found::\n\n";

	    for (i = 0; i < FoundFile.size(); i++) {

		Temp += "  ";
		Temp += FoundFile[i] + "\n";

	    }

	    throw runtime_error(Temp);

	}

    }

    LoadConfigFile(ProvisionalConfigFile);

}

void
UtilityConfig::LoadConfigFile(std::string Name) {

    SetConfigFileName(Name);

    if (Verbose()) {

	cout << "Using configuration file " << ConfigFile << ".\n";

    }

    // Load the data.
    try {

	CFGData.initializeFromFile(GetConfigFileName().c_str());

    } catch(...) {

	string Temp;

	Temp = "Error reading configuration file " + GetConfigFileName();
	Temp += ".";
	throw runtime_error(Temp);

    }

    if ( (CFGData.paths_workspace_path.length() == 0) ||
	 (CFGData.paths_rulebase_path.length() == 0) || 
	 (CFGData.paths_log_path.length() == 0) || 
	 (CFGData.update_script_call.length() == 0) || 
	 (CFGData.node_identity.length() == 0) ) {

	string Temp;

	Temp = "The configuration file " + GetConfigFileName();
	Temp += " did not have the necessary specification of one or more paths:\n";
	Temp += "\n  Workspace path:  " + CFGData.paths_workspace_path;
	Temp += "\n  Rulebase path:  " + CFGData.paths_rulebase_path;
	Temp += "\n  Log path:  " + CFGData.paths_log_path;
	Temp += "\n  Update script:  " + CFGData.update_script_call;
	Temp += "\n  Identity file:  " + CFGData.node_identity;
	throw runtime_error(Temp);

    }

}

string
UtilityConfig::GetPlatformContents(void) {

    return CFGData.PlatformElementContents;

}

string
UtilityConfig::GetConfigFileName(void) {

    return ConfigFile;

}

void
UtilityConfig::SetConfigFileName(string Name) {

    ConfigFile = Name;

}

string
UtilityConfig::GetWorkspacePath(void) {

    return CFGData.paths_workspace_path;

}

string
UtilityConfig::GetRulebasePath(void) {

    return CFGData.paths_rulebase_path;

}

string
UtilityConfig::GetLogPath(void) {

    return CFGData.paths_log_path;

}

string
UtilityConfig::GetIdentityFileName(void) {

    return CFGData.node_identity;

}

string
UtilityConfig::GetRulebaseScriptName(void) {

    return CFGData.update_script_call;

}

void
UtilityConfig::StartOrRestartMta(std::string Mta) {

    if (Verbose()) {

	cout << "Restarting the " << Mta << " MTA...";

    }

    if (!Explain()) {

	cout << "restarting the MTA is not implemented...";

    }

    OutputVerboseEnd();

}

void
UtilityConfig::LoadInfo(){

    if ("OpenBSD" == OperatingSystemType) {

	OsSpec = OpenBSD;
	PostfixMainCfPath = "/usr/local/etc/postfix/main.cf";
	PostfixMasterCfPath = "/usr/local/etc/postfix/master.cf";

    } else if ("FreeBSD" == OperatingSystemType) {

	OsSpec = FreeBSD;
	PostfixMainCfPath = "/etc/postfix/main.cf";
	PostfixMasterCfPath = "/etc/postfix/master.cf";

    } else if ("Ubuntu" == OperatingSystemType) {

	OsSpec = Ubuntu;
	PostfixMainCfPath = "/etc/postfix/main.cf";
	PostfixMasterCfPath = "/etc/postfix/master.cf";

    } else if ("RedHat" == OperatingSystemType) {

	OsSpec = RedHat;
	PostfixMainCfPath = "/etc/postfix/main.cf";
	PostfixMasterCfPath = "/etc/postfix/master.cf";

    } else if ("Suse" == OperatingSystemType) {

	OsSpec = Suse;
	PostfixMainCfPath = "/etc/postfix/main.cf";
	PostfixMasterCfPath = "/etc/postfix/master.cf";

    } else {

	ostringstream Temp;

	Temp << "Internal error in UtilityConfig::GetOsSpec:  Invalid value of OperatingSystemType:  "
	     << OperatingSystemType;

	throw runtime_error(Temp.str());

    }

}

void
UtilityConfig::UpdateIgnoreListFile() {

    string IgnoreListPath = GetWorkspacePath();

    string IgnoreListFile;

    IgnoreListFile = IgnoreListPath + DirectorySeparator;
    IgnoreListFile += "GBUdbIgnoreList.txt";

    SaveFile.CreateBackupFile(IgnoreListFile);                                  //  Save any existing file.

    if (!FileExists(IgnoreListFile)) {

        Copy(SampleIgnoreListFile, IgnoreListFile);                             // Use SNFMilter.xml.sample.

    }

    SetMode(IgnoreListFile, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);	        // Set permissions.
    SetOwnerGroup(IgnoreListFile);                                 	        // Set to sniffer user.

}

void
UtilityConfig::UpdateLogDir() {

    string LogDir = GetLogPath();

    if (!FileExists(LogDir)) {

        MkDir(LogDir);

    }

    SetMode(LogDir, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
    SetOwnerGroup(LogDir);

}

bool 
UtilityConfig::ProcessCommandLineItem(std::string OneInput) {

    bool ValidCommand = false;

    if (OneInput == VerboseKey) {

	SetVerbose(true);
	ValidCommand = true;

    } else if (OneInput == ExplainKey) {

	SetExplain(true);
	ValidCommand = true;

    } else if (OneInput == HelpKey) {

	SetHelp(true);
	ValidCommand = true;

    }

    return ValidCommand;

}

std::string
UtilityConfig::HelpCommandLine() {

    std::string Help;

    Help = "[ " + VerboseKey + " | " + ExplainKey + " ]";

    return Help;

}

std::string
UtilityConfig::HelpDescription() {

    std::string Desc;

    Desc  = "  -v                         Provide verbose output\n";
    Desc += "  -explain                   Provide an explaination of the actions\n";
    Desc += "                             without executing any commands\n";

    return Desc;

}
